/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt
 *
 * de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.nw.vrz.Fg254HeuschBoesefeldt wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */



package de.bsvrz.kex.tls.osi7.conversion.nw.vrz;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * Umsetzungsmodul für die FG 254, wird für die WanCom-Kopplung der ZDF basierten VRZ Leverkusen und VRZ Recklinghausen
 * mit der DaV basierten VRZ NRW verwendet.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision: 579 $ / $Date: 2012-03-16 14:27:54 +0100 (Fr, 16 Mrz 2012) $ / ($Author: HCK $)
 */
public class Fg254HeuschBoesefeldt extends FgAllHeuschBoesefeldt {

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Sonstige ---------------------------------------------------------------

    /**
     * DebugLogger für Debug-Ausgaben
     */
    private static final Debug debug = Debug.getLogger();

    // Private ----------------------------------------------------------------

    //~ FELDER ================================================================

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
     * Empfangsanmeldung als Empfänger.</li> <li> In der Methode {@link #update} wird für jede zur
     * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
     * wie folgt gebildet wird (hier für den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthält <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Für jede
     * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere passende Eintrage
     * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere ATG/ASP-Kombinationen in
     * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
     * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
    };

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
     * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block passende
     * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier für
     * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
     * DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
     * De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende {@link
     * DataDescription} für die zu sendenden Daten.</li> <li> Für jede Attributguppen/Aspekt
     * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
     * mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
     * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
     * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
        { "atg.tlsGloDeFehler", "asp.tlsAntwort", "254", "1", "1", "typ.deSys" }, { "atg.tlsGloDeFehler", "asp.tlsAntwort", "254", "131", "1", "typ.deSys" },  // DaSysFehlerDUE von Heusch/Boesefeldt
        { "atg.tlsGloKanalSteuerung", "asp.parameterIst", "255", "29", "2", "typ.deSys" },

        // {"atg.tlsGloGeographischeKenndaten", "asp.tlsZustand", "255", "36", "3", "typ.deSys"},
    };

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /**
     * Erzeugt ein Objekt vom Typ Fg001Default
     *
   	 * @param daf DaV
   	 * @param eak Eak
   	 * @param de De
   	 */
    public Fg254HeuschBoesefeldt(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    //~ METHODEN ==============================================================

    /**
     * FG 254 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Verwendet globale Behandlung De-Fehler in {@link
     * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg254Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        // Über die FG 254 werden gestörte SM oder EAK gemeldet. Für diese werden alle angemeldeten Daten an den
        // zugeordneten DE auf "keine Daten" gesetzt, da deren Zustand unbekannt ist.
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if (DEFehlerStatus != null) {
            String DEFehlerStatusName = DEFehlerStatus.getName();

            debug.fine("DEFehlerStatus: " + DEFehlerStatusName);

            if (DEFehlerStatusName.equals("StörSM")) {
                int      deChannel           = _de.getDeChannel();
                String   dePid               = _de.getDe().getPid();
                String[] excludedAtgAspArray = { "atg.tlsWzgBetriebsArt:asp.tlsVorgabeBedienung", dePid + ":atg.tlsGloDeFehler:asp.tlsAntwort" };

                debug.info("Fg254HeuschBoesefeldt: DE-Fehler mit " + DEFehlerStatusName + " für " + dePid + ", also für " + ((deChannel == 0) ? "SM selbst" : ("EAK " + deChannel)));

                if (deChannel == 0) {

                    // SM gestört, Knotennummer ermitteln.
                    int[] osi7Adress = new int[] { deBlock.getOsi7Adress() };

                    TlsApplicationLayer.writeNoDataToUnreachableDe(osi7Adress, _eakConversionModul.getTimeStamp(), excludedAtgAspArray);
                } else {                             // EAK gestört
                    Sm  sm  = _eak.getSm();
                    Eak eak = sm.getEak(deChannel);  // Eak mit der gesuchten Osi2Adresse am Sm ermitteln.

                    if (eak != null) {
                        TlsApplicationLayer.writeNoDataToUnreachableDe(eak, _eakConversionModul.getTimeStamp(), -1, excludedAtgAspArray);
                    }
                }
            }
        }

        return resultData;
    }

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * FG 254 : Typ: 131 Id: 1 Name: DUE-Fehler (Antwort). Verwendet globale Behandlung De-Fehler in {@link
     * #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock, Data, DataDescription)} und {@link #Fg254Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock, Data, DataDescription)}.
     * <p>
     * Interpretiert den DUE Fehler (Heusch/Boesefeldt spezifisch) genauso wie einen DeFehler der FG 254 mit DE = 0). De
     * keine eigene ATG für die Ergebnisspeicherung existiert, wird formal die der DeFehler FG254 verwendet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg254Typ131Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = Fg254Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);

        debug.fine("DUE Störung für Gerät erhalten;\n" + _eak.getSm().toString());

        // Ergebnisdatensatz wird nicht gespeichert!!!
        return resultData;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.09.15
